/*
 *  Solar System
 *  'm' cycles object
 *  'a' to toggle axes
 *  '0' snaps angles to 0,0
 *  arrows to rotate the world
 *  PgUp/PgDn zooms in/out
 */
#include "CSCIx229.h"
int mode;      //  Display mode
int axes=0;    //  Display axes
int th=0;      //  Azimuth of view angle
int ph=0;      //  Elevation of view angle
int zh=0;      //  Rotation
int fov=30;    //  Field of View
double asp=1;  //  Aspect ratio
double dim=3;  //  Size of world
GLUquadricObj* ball=NULL;

typedef struct
{
   char* name;
   char* ball;
   char* ring;
   double  R;
   double rot;
   double OrbitR;
   double OrbitT;
   double Incl;
   double Ecc;
   double th;
   double ph;
   unsigned int balltex;
   unsigned int ringtex;
}  Planet;
const int n=11;
Planet planet[] = {
//   Name      Texture      RingTexture        Radius   Rotation      Orbit(km) Orbit(d)  Incl  Ecc                
   {"Sun"    ,"sun.bmp"    ,NULL             , 695000 ,    24.60 ,          0.0,    1.00, 0.00,0.00,    0.0,   0.0, 0,0},
   {"Mercury","mercury.bmp",NULL             ,   2440 ,    58.60 ,   57910000.0,   87.97, 7.00,0.21,    0.0,   0.0, 0,0},
   {"Venus"  ,"venus.bmp"  ,NULL             ,   6052 ,  -243.00 ,  108200000.0,  224.70, 3.39,0.01, -180.0,   0.0, 0,0},
   {"Earth"  ,"earth.bmp"  ,NULL             ,   6378 ,     0.99 ,  149600000.0,  365.26, 0.00,0.02,  -23.5,   0.0, 0,0},
   {"Moon"   ,"moon.bmp"   ,NULL             ,   1738 ,     0.00 ,     384000.0,   27.32, 5.14,0.05,   -6.6,   0.0, 0,0},
   {"Mars"   ,"mars.bmp"   ,NULL             ,   3397 ,     1.03 ,  227940000.0,  686.98, 1.85,0.09,  -25.2,   0.0, 0,0},
   {"Jupiter","jupiter.bmp",NULL             ,  71492 ,     0.41 ,  778330000.0, 4332.71, 1.31,0.05,   -3.1,   0.0, 0,0},
   {"Saturn" ,"saturn.bmp" ,"saturnrings.bmp",  60268 ,     0.45 , 1429400000.0,10759.50, 2.49,0.06,  -26.7,  10.0, 0,0},
   {"Uranus" ,"uranus.bmp" ,"uranusrings.bmp",  25559 ,    -0.72 , 2870990000.0,30685.00, 0.77,0.05,    0.0,  98.0, 0,0},
   {"Neptune","neptune.bmp",NULL             ,  24766 ,     0.67 , 4504300000.0,60190.00, 1.77,0.01,  -29.0,   0.0, 0,0},
   {"Pluto"  ,"pluto.bmp"  ,NULL             ,   1150 ,    -6.39 , 5913520000.0,90550.00,17.15,0.25,    0.0,  90.0, 0,0},
   };

/*
 *  Draw planet
 */
void DrawPlanet(int n)
{
   glColor3f(1,1,1);
   glEnable(GL_TEXTURE_2D);
   //  Surface texture
   glBindTexture(GL_TEXTURE_2D,planet[n].balltex);
   //  Solid
   gluQuadricDrawStyle(ball,GLU_FILL);
   //  Calculate normals
   gluQuadricNormals(ball,GLU_SMOOTH);
   //  Apply Textures
   gluQuadricTexture(ball,1);
   //  Draw the ball
   gluSphere(ball,1.0,36,36);

   /*
    *  Draw rings for planets that have them
    *  We naively assume the ring width equals the radius
    */
   if (planet[n].ring)
   {
      int k;
      //  Make rings transparent grey
      glEnable(GL_BLEND);
      glBlendFunc(GL_ONE,GL_ONE_MINUS_SRC_COLOR);
      glColor3f(0.5,0.5,0.5);
      //  Set ring texture
      glBindTexture(GL_TEXTURE_2D,planet[n].ringtex);
      //  Draw ring plane
      glBegin(GL_QUAD_STRIP);
      for (k=0;k<=360;k+=2)
      {
         glTexCoord2f(1,0);glVertex2d(1.0*Cos(k),1.0*Sin(k));
         glTexCoord2f(0,0);glVertex2d(2.2*Cos(k),2.2*Sin(k));
      }
      glEnd();
      glDisable(GL_BLEND);
   }
}

/*
 *  OpenGL (GLUT) calls this routine to display the scene
 */
void display()
{
   //  Length of axes
   const double len=1.2;
   //  Eye position
   double Ex = -2*dim*Cos(ph);
   double Ey = +2*dim*Sin(ph);
   double Ez = 0;
   //  Erase the window and the depth buffer
   glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
   //  Set perspective
   glLoadIdentity();
   gluLookAt(Ex,Ey,Ez , 0,0,0 , 0,Cos(ph),0);
   //  Draw scene
   glEnable(GL_DEPTH_TEST);
   //  Rotate Z up
   glRotated(-90,1,0,0);

   /*
    * Draw solar system
    */
   if (mode<0)
   {
      int k;
      glRotated(th,0,0,1);            //  View angle
      glScaled(6.7e-9,6.7e-9,6.7e-9); //  Convert km to AU
      for (k=0;k<11;k++)
      {
         double d = 1000*zh;      //  Time   (days)
         double r = planet[k].R;  //  Radius (km)
         if (k==4) continue;      // Skip the moon
         glPushMatrix();
         glRotated(d/planet[k].OrbitT,0,0,1); //  Rotation around sun
         glTranslated(planet[k].OrbitR,0,0);  //  Translate to orbit
         glRotated(planet[k].th,0,1,0);       //  Declination
         glRotated(d/planet[k].rot,0,0,1);    //  Rotation around axis
         glScaled(r,r,r);                     //  Radius of planet
         DrawPlanet(k);                       //  Draw planet
         glPopMatrix();
      }
   }
   /*
    *  Draw planet
    */
   else
   {
     glRotated(th,1,0,0);  // Declination
     glRotated(zh,0,0,1);  // Spin around axes
     DrawPlanet(mode);
   }

   /*
    *  Draw axes - no textures from here
    */
   glDisable(GL_TEXTURE_2D);
   glColor3f(1,1,1);
   if (axes)
   {
      glBegin(GL_LINES);
      glVertex3d(0.0,0.0,0.0);
      glVertex3d(len,0.0,0.0);
      glVertex3d(0.0,0.0,0.0);
      glVertex3d(0.0,len,0.0);
      glVertex3d(0.0,0.0,0.0);
      glVertex3d(0.0,0.0,len);
      glEnd();
      //  Label axes
      glRasterPos3d(len,0.0,0.0);
      Print("X");
      glRasterPos3d(0.0,len,0.0);
      Print("Y");
      glRasterPos3d(0.0,0.0,len);
      Print("Z");
   }
   //  Display parameters
   glWindowPos2i(5,5);
   Print("Angle=%d,%d  FoV=%d Object=%s",th,ph,fov,mode<0?"Solar System":planet[mode].name);
   //  Render the scene and make it visible
   glFlush();
   glutSwapBuffers();
}

/*
 *  GLUT calls this routine when an arrow key is pressed
 */
void special(int key,int x,int y)
{
   //  Right arrow key - increase angle by 5 degrees
   if (key == GLUT_KEY_RIGHT)
      th += 5;
   //  Left arrow key - decrease angle by 5 degrees
   else if (key == GLUT_KEY_LEFT)
      th -= 5;
   //  Up arrow key - increase elevation by 5 degrees
   else if (key == GLUT_KEY_UP)
      ph += 5;
   //  Down arrow key - decrease elevation by 5 degrees
   else if (key == GLUT_KEY_DOWN)
      ph -= 5;
   //  PageUp key - increase FOV
   else if (key == GLUT_KEY_PAGE_DOWN && fov<179)
      fov++;
   //  PageDown key - decrease FOV
   else if (key == GLUT_KEY_PAGE_UP && fov>1)
      fov--;
   //  Keep angles to +/-360 degrees
   th %= 360;
   ph %= 360;
   //  Update projection
   Project(fov,asp,dim);
   //  Tell GLUT it is necessary to redisplay the scene
   glutPostRedisplay();
}

void SetMode(int k)
{
   mode = k;
   th = planet[k].th;
   ph = planet[k].ph;
}

/*
 *  GLUT calls this routine when a key is pressed
 */
void key(unsigned char ch,int x,int y)
{
   //  Exit on ESC
   if (ch == 27)
      exit(0);
   //  Reset view angle
   else if (ch == '0')
   {
      th = planet[mode].th;
      ph = planet[mode].ph;
   }
   //  Select display mode
   else if (ch == 'm')
      SetMode((mode+1)%n);
   else if (ch == 'M')
      SetMode((mode+n-1)%n);
   else if (ch == 'v')
      mode = -1;
   //  Toggle axes
   else if (ch == 'a' || ch == 'A')
      axes = 1-axes;
   //  Reproject
   Project(fov,asp,dim);
   //  Tell GLUT it is necessary to redisplay the scene
   glutPostRedisplay();
}

/*
 *  GLUT calls this routine when the window is resized
 */
void reshape(int width,int height)
{
   //  Ratio of the width to the height of the window
   asp = (height>0) ? (double)width/height : 1;
   //  Set the viewport to the entire window
   glViewport(0,0, width,height);
   //  Set projection
   Project(fov,asp,dim);
}

/*
 *  GLUT calls this routine when the window is resized
 */
void idle()
{
   zh = 15*glutGet(GLUT_ELAPSED_TIME)/1000.0;
   glutPostRedisplay();
}

/*
 *  Start up GLUT and tell it what to do
 */
int main(int argc,char* argv[])
{
   int k;
   //  Initialize GLUT
   glutInit(&argc,argv);
   //  Request double buffered, true color window with Z buffering at 600x600
   glutInitDisplayMode(GLUT_RGB | GLUT_DEPTH | GLUT_DOUBLE | GLUT_ALPHA);
   glutCreateWindow("Solar System");
   glutFullScreen();
   //  Set callbacks
   glutDisplayFunc(display);
   glutReshapeFunc(reshape);
   glutSpecialFunc(special);
   glutKeyboardFunc(key);
   glutIdleFunc(idle);
   //  Allocate quadric for ball
   ball = gluNewQuadric();
   //  Load textures
   for (k=0;k<n;k++)
   {
      planet[k].balltex = LoadTexBMP(planet[k].ball);
      planet[k].ringtex = planet[k].ring ? LoadTexBMP(planet[k].ring) : 0;
   }
   SetMode(3);
   //  Pass control to GLUT so it can interact with the user
   glutMainLoop();
   return 0;
}
